#include <iostream>
#include <fstream>
#include <string>
#include <boost/format.hpp>

#define DEPTH_WIDTH    1280
#define DEPTH_HEIGHT    800

#define FOCAL_LENGTH_X 835.2802734375
#define FOCAL_LENGTH_Y 835.9163208008
#define PRINCIPAL_X    409.8193359375
#define PRINCIPAL_Y    636.9195556641

extern "C" void SaveXyz(unsigned short* depthBuf, int frmCnt){
	std::fstream fs;
	std::string filename = "/home/himax/" + std::to_string(frmCnt) + ".xyz";
	std::string str = "";
	std::string tmp = "%.2f %.2f %.2f\n";

	double Kd[3][3] =
	{
		{ FOCAL_LENGTH_Y, 0, PRINCIPAL_Y },
		{ 0, FOCAL_LENGTH_X, PRINCIPAL_X },
		{ 0, 0, 1 }
	};

	double det = Kd[0][0] * (Kd[1][1] * Kd[2][2] - Kd[2][1] * Kd[1][2]) -
		Kd[0][1] * (Kd[1][0] * Kd[2][2] - Kd[1][2] * Kd[2][0]) +
		Kd[0][2] * (Kd[1][0] * Kd[2][1] - Kd[1][1] * Kd[2][0]);

	double invdet = 1 / det;
	double Kd_[3][3];
	Kd_[0][0] = (Kd[1][1] * Kd[2][2] - Kd[2][1] * Kd[1][2]) * invdet;
	Kd_[0][1] = (Kd[0][2] * Kd[2][1] - Kd[0][1] * Kd[2][2]) * invdet;
	Kd_[0][2] = (Kd[0][1] * Kd[1][2] - Kd[0][2] * Kd[1][1]) * invdet;
	Kd_[1][0] = (Kd[1][2] * Kd[2][0] - Kd[1][0] * Kd[2][2]) * invdet;
	Kd_[1][1] = (Kd[0][0] * Kd[2][2] - Kd[0][2] * Kd[2][0]) * invdet;
	Kd_[1][2] = (Kd[1][0] * Kd[0][2] - Kd[0][0] * Kd[1][2]) * invdet;
	Kd_[2][0] = (Kd[1][0] * Kd[2][1] - Kd[2][0] * Kd[1][1]) * invdet;
	Kd_[2][1] = (Kd[2][0] * Kd[0][1] - Kd[0][0] * Kd[2][1]) * invdet;
	Kd_[2][2] = (Kd[0][0] * Kd[1][1] - Kd[1][0] * Kd[0][1]) * invdet;

	for (int j = 0; j<DEPTH_HEIGHT; j++){
		for (int i = 0; i<DEPTH_WIDTH; i++){
			int loc = j*DEPTH_WIDTH + i;
			double realdepth = depthBuf[loc] / 16; // Unit mm

			double xz = Kd_[0][0] * i + Kd_[0][1] * j + Kd_[0][2];
			double yz = Kd_[1][0] * i + Kd_[1][1] * j + Kd_[1][2];
			double zz = Kd_[2][0] * i + Kd_[2][1] * j + Kd_[2][2];

			double Xnir = xz / zz*realdepth;
			double Ynir = yz / zz*realdepth;
			double Znir = realdepth;

			if (Xnir != 0 || Ynir != 0 || Znir != 0)
			{
				str += boost::str(boost::format(tmp) % Xnir % Ynir % Znir);
			}
		}
	}

	fs.open(filename, std::ios_base::out);
	if (fs.is_open()){
    		fs.write(str.data(), str.size());
		std::cout << "save xyz: " << filename << std::endl;
	}
	
	fs.close();
}

